@@ -1,7 +1,7 @@
 
 setMethod("length", signature(x="SpatRasterDataset"),
 	function(x) {
-		x@cpp$nsds()
+		x@ptr$nsds()
 	}
 )
 
@@ -30,7 +30,7 @@ setMethod("sds", signature(x="character"),
 		}
 		if (is.null(opts)) opts <- ""[0]
 		if (raw) opts <- c(opts, "so=false")		
-		r@cpp <- SpatRasterStack$new(f, ids, useids, opts)
+		r@ptr <- SpatRasterStack$new(f, ids, useids, opts)
 		messages(r, "sds")
 	}
 )
@@ -38,7 +38,7 @@ setMethod("sds", signature(x="character"),
 setMethod("sds", signature(x="missing"),
 	function(x) {
 		r <- methods::new("SpatRasterDataset")
-		r@cpp <- SpatRasterStack$new()
+		r@ptr <- SpatRasterStack$new()
 		r
 	}
 )
@@ -47,8 +47,8 @@ setMethod("sds", signature(x="missing"),
 setMethod("sds", signature(x="SpatRaster"),
 	function(x, ...) {
 		r <- methods::new("SpatRasterDataset")
-		r@cpp <- SpatRasterStack$new()
-		r@cpp$add(x@cpp, varnames(x)[1], longnames(x)[1], units(x)[1], FALSE)
+		r@ptr <- SpatRasterStack$new()
+		r@ptr$add(x@ptr, varnames(x)[1], longnames(x)[1], units(x)[1], FALSE)
 		dots <- list(...)
 		nms <- names(dots)
 		if (is.null(nms)) nms = ""
@@ -57,7 +57,7 @@ setMethod("sds", signature(x="SpatRaster"),
 			if (inherits(dots[[i]], "SpatRaster")) {
 				vname <- nms[i]
 				if (vname == "") vname = varnames(dots[[i]])[1]
-				r@cpp$add(dots[[i]]@cpp, vname, longnames(dots[[i]])[1], units(dots[[i]])[1], FALSE)
+				r@ptr$add(dots[[i]]@ptr, vname, longnames(dots[[i]])[1], units(dots[[i]])[1], FALSE)
 			}
 		}
 		messages(r, "sds")
@@ -67,16 +67,16 @@ setMethod("sds", signature(x="SpatRaster"),
 setMethod("sds", signature(x="list"),
 	function(x) {
 		r <- methods::new("SpatRasterDataset")
-		r@cpp <- SpatRasterStack$new()
+		r@ptr <- SpatRasterStack$new()
 		nms <- names(x)
 		if (is.null(nms)) nms <- rep("", length(x))
 		for (i in seq_along(x)) {
 			if (inherits(x[[i]], "SpatRaster")) {
-				r@cpp$add(x[[i]]@cpp, nms[i], "", "", FALSE)
+				r@ptr$add(x[[i]]@ptr, nms[i], "", "", FALSE)
 			} else if (inherits(x[[i]], "SpatRasterDataset")) {
 				y <- as.list(x[[i]])
 				ynms <- names(x[[i]])
-				s <- sapply(y, function(j) r@cpp$add(j@cpp, ynms[j], "", "", FALSE))
+				s <- sapply(y, function(j) r@ptr$add(j@ptr, ynms[j], "", "", FALSE))
 			} else {
 				name <- names(x[[i]])
 				cls <- paste(class(x[[i]]), collapse=", ")
@@ -139,7 +139,7 @@ setMethod("sds", signature(x="stars_proxy"),
 setMethod("c", signature(x="SpatRasterDataset"),
 	function(x, ...) {
 
-		x@cpp <- x@cpp$subset((1:x@cpp$nsds()) -1 ) # why? make a copy?
+		x@ptr <- x@ptr$subset((1:x@ptr$nsds()) -1 ) # why? make a copy?
 
 		dots <- list(...)
 		nms <- names(dots)
@@ -148,14 +148,14 @@ setMethod("c", signature(x="SpatRasterDataset"),
 			if (inherits(dots[[i]], "SpatRasterDataset")) {
 				sdsnms <- names(dots[[i]])
 				for (j in 1:(length(dots[[i]]))) {
-					if (!x@cpp$add(dots[[i]][[j]]@cpp, sdsnms[j], "", "", FALSE)) {
+					if (!x@ptr$add(dots[[i]][[j]]@ptr, sdsnms[j], "", "", FALSE)) {
 						messages(x, "c")
 					}
 				}
 
 			} else if (inherits(dots[[i]], "SpatRaster")) {
 				if (is.null(nms)) error("c", "arguments must be named")
-				if (!x@cpp$add(dots[[i]]@cpp, nms[i], "", "", FALSE)) {
+				if (!x@ptr$add(dots[[i]]@ptr, nms[i], "", "", FALSE)) {
 					messages(x, "c")
 				}
 			} else {
@@ -176,9 +176,9 @@ setReplaceMethod("[", c("SpatRasterDataset", "numeric", "missing"),
 		i <- sort(i)
 		for (j in i) {
 			if (j == (length(x)+1)) {
-				x@cpp$add(value@cpp, "", "", "", FALSE)
+				x@ptr$add(value@ptr, "", "", "", FALSE)
 			} else {
-				x@cpp$replace(j-1, value@cpp)
+				x@ptr$replace(j-1, value@ptr)
 			}
 		}
 		messages(x, "`[`")
@@ -191,11 +191,11 @@ function(x, i, j, drop=TRUE) {
 	i <- positive_indices(i, length(x), TRUE, "`[`(i)")
 
 	if (drop && (length(i) == 1)) {
-		ptr <- x@cpp$getsds(i-1)
+		ptr <- x@ptr$getsds(i-1)
 		x <- rast()
-		x@cpp <- ptr
+		x@ptr <- ptr
 	} else {
-		x@cpp <- x@cpp$subset(i-1)
+		x@ptr <- x@ptr$subset(i-1)
 	}
 	messages(x, "`[`")
 })
@@ -210,10 +210,10 @@ function(x, i, j, drop=TRUE) {
 		out <- rast(out)
 	} else {
 		out <- sds()
-		nms <- x@cpp$names
+		nms <- x@ptr$names
 		for (k in nd) {
 			r <- x[k][[j]]
-			out@cpp$add(r@cpp, nms[k], "", "", FALSE)
+			out@ptr$add(r@ptr, nms[k], "", "", FALSE)
 		}
 	}
 	messages(out, "`[`")
@@ -227,13 +227,13 @@ function(x, i, j, drop=TRUE) {
 
 setMethod("[", c("SpatRasterDataset", "missing", "numeric"),
 function(x, i, j, drop=TRUE) {
-	`[`(x, i=1:x@cpp$nsds(), j=j, drop=drop)
+	`[`(x, i=1:x@ptr$nsds(), j=j, drop=drop)
 })
 
 setMethod("[", c("SpatRasterDataset", "missing", "logical"),
 function(x, i, j, drop=TRUE) {
 	j <- positive_indices(j, min(nlyr(x)))
-	`[`(x, i=1:x@cpp$nsds(), j=j, drop=drop)
+	`[`(x, i=1:x@ptr$nsds(), j=j, drop=drop)
 })
 
 
@@ -279,7 +279,7 @@ setMethod("$", "SpatRasterDataset",
 setMethod("sprc", signature(x="missing"),
 	function(x) {
 		r <- methods::new("SpatRasterCollection")
-		r@cpp <- SpatRasterCollection$new()
+		r@ptr <- SpatRasterCollection$new()
 		r
 	}
 )
@@ -299,11 +299,11 @@ setMethod("sprc", signature(x="list"),
 		if (n > 0) {
 			for (i in 1:n) {
 				if (inherits(x[[i]], "SpatRaster")) {
-					ptr$add(x[[i]]@cpp, "")
+					ptr$add(x[[i]]@ptr, "")
 				} else if (inherits(x[[i]], "SpatRasterCollection") | 
 							inherits(x[[i]], "SpatRasterDataset")) {
 					y <- as.list(x[[i]])
-					s <- sapply(y, function(j) ptr$add(j@cpp, ""))
+					s <- sapply(y, function(j) ptr$add(j@ptr, ""))
 				} else {
 					name <- names(x[[i]])
 					cls <- paste(class(x[[i]]), collapse=", ")
@@ -312,7 +312,7 @@ setMethod("sprc", signature(x="list"),
 			}
 		}
 		x <- new("SpatRasterCollection")
-		x@cpp <- ptr
+		x@ptr <- ptr
 		x
 	}
 )
@@ -342,7 +342,7 @@ setMethod("sprc", signature(x="character"),
 		if (is.null(opts)) opts <- ""[0]
 		if (raw) opts <- c(opts, "so=false")		
 		
-		r@cpp <- SpatRasterCollection$new(f, ids, useids, opts)
+		r@ptr <- SpatRasterCollection$new(f, ids, useids, opts)
 		messages(r, "sprc")
 	}
 )
@@ -350,7 +350,7 @@ setMethod("sprc", signature(x="character"),
 
 setMethod("length", signature(x="SpatRasterCollection"),
 	function(x) {
-		x@cpp$length()
+		x@ptr$length()
 	}
 )
 
@@ -358,16 +358,16 @@ setMethod("[", c("SpatRasterCollection", "numeric", "missing"),
 function(x, i, j, ... ,drop=TRUE) {
 	i <- positive_indices(i, length(x), TRUE, "`[`(i)")
 	if (drop && (length(i) == 1)) {
-		ptr <- x@cpp$x[[i]]
+		ptr <- x@ptr$x[[i]]
 		x <- rast()
-		x@cpp <- ptr
+		x@ptr <- ptr
 	} else {
-		s <- x@cpp$x[i]
+		s <- x@ptr$x[i]
 		ptr <- SpatRasterCollection$new()
 		for (i in 1:length(s)) {
 			ptr$add(s[[i]], "")
 		}
-		x@cpp <- ptr
+		x@ptr <- ptr
 	}
 	messages(x, "`[`")
 })
@@ -375,7 +375,7 @@ function(x, i, j, ... ,drop=TRUE) {
 
 setMethod("add<-", signature("SpatRasterCollection", "SpatRaster"),
 	function(x, value) {
-		x@cpp$add(value@cpp, "")
+		x@ptr$add(value@ptr, "")
 		messages(x, "add")
 	}
 )
@@ -383,7 +383,7 @@ setMethod("add<-", signature("SpatRasterCollection", "SpatRaster"),
 
 setMethod("add<-", signature("SpatRasterDataset", "SpatRaster"),
 	function(x, value) {
-		x@cpp$add(value@cpp, "")
+		x@ptr$add(value@ptr, "")
 		messages(x, "add")
 	}
 )
@@ -400,7 +400,7 @@ setMethod("c", signature(x="SpatRasterCollection"),
 setMethod("c", signature(x="SpatRasterCollection"),
 	function(x, ...) {
 
-		x@cpp <- x@cpp$deepcopy()
+		x@ptr <- x@ptr$deepcopy()
 
 		dots <- list(...)
 		nms <- names(dots)
@@ -408,7 +408,7 @@ setMethod("c", signature(x="SpatRasterCollection"),
 			if (inherits(dots[[i]], "SpatRasterCollection") | inherits(dots[[i]], "SpatRasterDataset")) {
 				sdsnms <- names(dots[[i]])
 				for (j in 1:(length(dots[[i]]))) {
-					x@cpp$add(dots[[i]][[j]]@cpp, sdsnms[j])
+					x@ptr$add(dots[[i]][[j]]@ptr, sdsnms[j])
 				}
 			} else if (inherits(dots[[i]], "SpatRaster")) {
 				if (is.null(nms)) { 
@@ -416,7 +416,7 @@ setMethod("c", signature(x="SpatRasterCollection"),
 				} else { 
 					name <- nms[i] 
 				}
-				x@cpp$add(dots[[i]]@cpp, name)
+				x@ptr$add(dots[[i]]@ptr, name)
 			} else {
 				error("c", "arguments must be SpatRaster, SpatRasterCollection, or SpatRasterDataset")
 			}
