@@ -3,52 +3,9 @@
 # Version 1.0
 # License GPL v3
 
-setMethod("metags", signature(x="SpatRaster"),
-	function(x, name=NULL) {
-		v <- x@cpp$getTags()
-		m <- matrix(v, ncol=2, byrow=TRUE, dimnames = list(NULL, c("name", "value")))
-		out <- m[,2]
-		names(out) <- m[,1]
-		if (!is.null(name)) {
-			out <- out[name]
-		} 
-		out
-	}
-)
-
-
-setMethod("metags<-", signature(x="SpatRaster"),
-	function(x, value) {
-		if (is.null(value)) {
-			value <- matrix(x@cpp$getTags(), ncol=2, byrow=TRUE)
-			value[,2] <- ""
-		} else if (NCOL(value) == 1) {
-			value <- strsplit(value, "=")
-			i <- sapply(value, length) == 1
-			if (length(i) > 0) {
-				j <- which(i)
-				for (i in j) value[[i]] <- c(value[[i]], "")
-			}
-			i <- sapply(value, length) == 2
-			value <- do.call(rbind, value[i])
-		} else if (NCOL(value) != 2) {
-			error("metags<-", "expecting a vector with 'name=value' or a two column matrix")
-		}
-		value[is.na(value)] <- ""
-		x <- deepcopy(x)
-		if (NROW(value) > 0) {
-			sapply(1:nrow(value), function(i) {
-				x@cpp$addTag(value[i,1], value[i,2])
-			})
-		}
-		x
-	}
-)
-
-
 setMethod("is.rotated", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$is_rotated()
+		x@ptr$is_rotated()
 	}
 )
 
@@ -56,29 +13,17 @@ setMethod("is.rotated", signature(x="SpatRaster"),
 setMethod("rangeFill", signature(x="SpatRaster"),
 	function(x, limit, circular=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$fill_range(limit, circular, opt)
+		x@ptr <- x@ptr$fill_range(limit, circular, opt)
 		messages(x, "rangeFill")
 	}
 )
 
-setMethod("meta", signature(x="SpatRaster"),
-	function(x, layers=FALSE) {
-		f <- function(i) {
-			if (length(i) == 0) {
-				matrix(ncol=2, nrow=0)
-			} else {
-				matrix(unlist(regmatches(i, regexpr("=", i), invert=TRUE)), ncol=2, byrow=TRUE)
-			}
-		}
-		lapply(x@cpp$metadata(layers), f)
-	}
-)
 
 
 setMethod("weighted.mean", signature(x="SpatRaster", w="numeric"),
 	function(x, w, na.rm=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$wmean_vect(w, na.rm, opt)
+		x@ptr <- x@ptr$wmean_vect(w, na.rm, opt)
 		messages(x, "weighted.mean")
 	}
 )
@@ -87,7 +32,7 @@ setMethod("weighted.mean", signature(x="SpatRaster", w="numeric"),
 setMethod("weighted.mean", signature(x="SpatRaster", w="SpatRaster"),
 	function(x, w, na.rm=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <-x@cpp$wmean_rast(w@cpp, na.rm, opt)
+		x@ptr <-x@ptr$wmean_rast(w@ptr, na.rm, opt)
 		messages(x, "weighted.mean")
 	}
 )
@@ -98,11 +43,11 @@ setMethod("patches", signature(x="SpatRaster"),
 	function(x, directions=4, zeroAsNA=FALSE, allowGaps=TRUE, filename="", ...) {
 		if (allowGaps) {
 			opt <- spatOptions(filename, ...)
-			x@cpp <- x@cpp$patches(directions[1], zeroAsNA[1], opt)
+			x@ptr <- x@ptr$patches(directions[1], zeroAsNA[1], opt)
 			messages(x, "patches")
 		} else {
 			opt <- spatOptions()
-			x@cpp <- x@cpp$patches(directions[1], zeroAsNA[1], opt)
+			x@ptr <- x@ptr$patches(directions[1], zeroAsNA[1], opt)
 			x <- messages(x, "patches")
 			f <- freq(x)
 			fr <- cbind(f[,2], 1:nrow(f))
@@ -120,7 +65,7 @@ setMethod("patches", signature(x="SpatRaster"),
 
 setMethod("origin", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$origin
+		x@ptr$origin
 	}
 )
 
@@ -158,7 +103,7 @@ setMethod("origin<-", signature("SpatRaster"),
 
 setMethod("align", signature(x="SpatExtent", y="SpatRaster"),
 	function(x, y, snap="near") {
-		x@cpp <- y@cpp$align(x@cpp, tolower(snap))
+		x@ptr <- y@ptr$align(x@ptr, tolower(snap))
 		#messages(x, "align")
 		x
 	}
@@ -166,7 +111,7 @@ setMethod("align", signature(x="SpatExtent", y="SpatRaster"),
 
 setMethod("align", signature(x="SpatExtent", y="numeric"),
 	function(x, y) {
-		x@cpp <- x@cpp$align(y, "")
+		x@ptr <- x@ptr$align(y, "")
 		x
 	}
 )
@@ -175,7 +120,7 @@ setMethod("align", signature(x="SpatExtent", y="numeric"),
 setMethod("intersect", signature(x="SpatRaster", y="SpatRaster"),
 	function(x, y) {
 		opt <- spatOptions() 
-		x@cpp <- x@cpp$intersect(y@cpp, opt)
+		x@ptr <- x@ptr$intersect(y@ptr, opt)
 		messages(x)
 	}
 )
@@ -185,7 +130,7 @@ setMethod("cellSize", signature(x="SpatRaster"),
 	function(x, mask=FALSE, lyrs=FALSE, unit="m", transform=TRUE, rcx=100, filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		if (!lyrs) x <- x[[1]]
-		x@cpp <- x@cpp$rst_area(mask, unit, transform, rcx, opt)
+		x@ptr <- x@ptr$rst_area(mask, unit, transform, rcx, opt)
 		messages(x, "cellSize")
 	}
 )
@@ -194,7 +139,7 @@ setMethod("cellSize", signature(x="SpatRaster"),
 setMethod("atan2", signature(y="SpatRaster", x="SpatRaster"),
 	function(y, x) {
 		opt <- spatOptions(filename="", overwrite=TRUE)
-		y@cpp <- y@cpp$atan2(x@cpp, opt)
+		y@ptr <- y@ptr$atan2(x@ptr, opt)
 		messages(y, "atan2")
 	}
 )
@@ -202,7 +147,7 @@ setMethod("atan2", signature(y="SpatRaster", x="SpatRaster"),
 setMethod("atan_2", signature(y="SpatRaster", x="SpatRaster"),
 	function(y, x, filename="", ...) {
 		opt <- spatOptions(filename=filename, ...)
-		y@cpp <- y@cpp$atan2(x@cpp, opt)
+		y@ptr <- y@ptr$atan2(x@ptr, opt)
 		messages(y, "atan_2")
 	}
 )
@@ -212,20 +157,20 @@ setMethod("boundaries", signature(x="SpatRaster"),
 	function(x, classes=FALSE, inner=TRUE, directions=8, falseval=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		type <- ifelse(inner[1], "inner", "outer")
-		x@cpp <- x@cpp$boundaries(classes[1], type, directions[1], falseval[1], opt)
+		x@ptr <- x@ptr$boundaries(classes[1], type, directions[1], falseval[1], opt)
 		messages(x, "boundaries")
 	}
 )
 
 
 .collapseSources <- function(x) {
-	x@cpp <- x@cpp$collapse_sources()
+	x@ptr <- x@ptr$collapse_sources()
 	messages(x, "tighten")
 }
 
 setMethod("deepcopy", signature("SpatRaster"),
 	function(x) {
-		x@cpp <- x@cpp$deepcopy()
+		x@ptr <- x@ptr$deepcopy()
 		x
 	}
 )
@@ -244,18 +189,18 @@ setMethod("split", signature(x="SpatRaster"),
 
 setMethod("add<-", signature("SpatRaster", "SpatRaster"),
 	function(x, value) {
-		if (x@cpp$same(value@cpp)) {
-			x@cpp <- x@cpp$deepcopy()
+		if (x@ptr$same(value@ptr)) {
+			x@ptr <- x@ptr$deepcopy()
 		}
 		opt <- spatOptions()
-		x@cpp$addSource(value@cpp, FALSE, opt)
+		x@ptr$addSource(value@ptr, FALSE, opt)
 		messages(x, "add")
 	}
 )
 
 setMethod("tighten", signature("SpatRaster"),
 	function(x) {
-		x@cpp <- x@cpp$collapse_sources()
+		x@ptr <- x@ptr$collapse_sources()
 		messages(x, "tighten")
 	}
 )
@@ -263,7 +208,7 @@ setMethod("tighten", signature("SpatRaster"),
 setMethod("tighten", signature("SpatRasterDataset"),
 	function(x) {
 		y <- new("SpatRaster")
-		y@cpp <- x@cpp$collapse()
+		y@ptr <- x@ptr$collapse()
 		messages(y, "tighten")
 	}
 )
@@ -272,9 +217,9 @@ setMethod("tighten", signature("SpatRasterDataset"),
 #setMethod("c", signature(x="SpatRaster"),
 #	function(x, ...) {
 #		s <- sds(list(x, ...))
-#		x@cpp <- s@cpp$collapse()
+#		x@ptr <- s@ptr$collapse()
 #		x <- messages(x, "c")
-#		try( x@cpp <- x@cpp$collapse_sources() )
+#		try( x@ptr <- x@ptr$collapse_sources() )
 #		messages(x, "c")
 #	}
 #)
@@ -283,12 +228,12 @@ setMethod("tighten", signature("SpatRasterDataset"),
 
 #cbind.SpatVector <- function(x, y, ...) {
 #	if (inherits(y, "SpatVector")) {
-#		y <- y@cpp$df
+#		y <- y@ptr$df
 #	} else {
 #		stopifnot(inherits(y, "data.frame"))
 #		y <- terra:::.makeSpatDF(y)
 #	}
-#	x@cpp <- x@cpp$cbind(y)
+#	x@ptr <- x@ptr$cbind(y)
 #	messages(x, "cbind")
 #}
 
@@ -296,12 +241,13 @@ cbind.SpatVector <- function(x, y, ...) {
 	dots <- list(y, ...)
 	for (y in dots) {
 		if (inherits(y, "SpatVector")) {
-			y <- y@cpp$df
+			y <- y@ptr$df
 		} else {
-			stopifnot(inherits(y, "data.frame"))
+			# stopifnot(inherits(y, "data.frame"))
+			y <- as.data.frame(y)
 			y <- .makeSpatDF(y)
 		}
-		x@cpp <- x@cpp$cbind(y)
+		x@ptr <- x@ptr$cbind(y)
 		x <- messages(x, "cbind")
 	}
 	x
@@ -312,18 +258,18 @@ rbind.SpatVector <- function(x, y, ...) {
 	if (missing(y) || is.null(y)) return(x)
 	stopifnot(inherits(y, "SpatVector"))
 	s <- svc(x, y, ...)
-	x@cpp <- s@cpp$append()
+	x@ptr <- s@ptr$append()
 	messages(x, "rbind")
 
 #	dots <- list(...)
 #	if (is.null(dots)) {
-#		x@cpp <- x@cpp$rbind(y@cpp, FALSE)
+#		x@ptr <- x@ptr$rbind(y@ptr, FALSE)
 #	} else {
 
 	#if (!is.null(dots)) {
 	#	for (y in dots) {
 	#		stopifnot(inherits(y, "SpatVector"))
-	#		x@cpp <- x@cpp$rbind(y@cpp, FALSE)
+	#		x@ptr <- x@ptr$rbind(y@ptr, FALSE)
 	#		x <- messages(x, "rbind")
 	#	}
 	#}
@@ -358,18 +304,18 @@ setMethod("clamp", signature(x="SpatRaster"),
 		rupp <- inherits(upper, "SpatRaster")
 		r <- rast()
 		if (rlow & rupp) {
-			x@cpp <- x@cpp$clamp_raster(lower@cpp, upper@cpp, NA, NA, values[1], opt)
+			x@ptr <- x@ptr$clamp_raster(lower@ptr, upper@ptr, NA, NA, values[1], opt)
 		} else if (rlow) {
 			if (any(is.na(upper))) error("clamp", "upper limit cannot be NA")
-			x@cpp <- x@cpp$clamp_raster(lower@cpp, r@cpp, NA, upper, values[1], opt)
+			x@ptr <- x@ptr$clamp_raster(lower@ptr, r@ptr, NA, upper, values[1], opt)
 		} else if (rupp) {
 			if (any(is.na(lower))) error("clamp", "lower limit cannot be NA")
-			x@cpp <- x@cpp$clamp_raster(r@cpp, upper@cpp, lower, NA, values[1], opt)
+			x@ptr <- x@ptr$clamp_raster(r@ptr, upper@ptr, lower, NA, values[1], opt)
 		} else {
 			if (any(is.na(lower))) error("clamp", "lower limit cannot be NA")
 			if (any(is.na(upper))) error("clamp", "upper limit cannot be NA")
-			x@cpp <- x@cpp$clamp_raster(r@cpp, r@cpp, lower, upper, values[1], opt)
-			#x@cpp <- x@cpp$clamp(lower, upper, values[1], opt)
+			x@ptr <- x@ptr$clamp_raster(r@ptr, r@ptr, lower, upper, values[1], opt)
+			#x@ptr <- x@ptr$clamp(lower, upper, values[1], opt)
 		}
 		messages(x, "clamp")
 	}
@@ -380,7 +326,7 @@ setMethod("clamp", signature(x="SpatRaster"),
 setMethod("clamp_ts", signature(x="SpatRaster"),
 	function(x, min=FALSE, max=TRUE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$clamp_ts(min, max, opt)
+		x@ptr <- x@ptr$clamp_ts(min, max, opt)
 		messages(x, "clamp_ts")
 	}
 )
@@ -420,7 +366,7 @@ function(x, rcl, include.lowest=FALSE, right=TRUE, others=NULL, brackets=TRUE, f
 		others <- TRUE
 	}
 	keepcats <- FALSE
-    x@cpp <- x@cpp$classify(as.vector(rcl), NCOL(rcl), right, include.lowest, others, othersValue, bylayer[1], brackets[1], keepcats, opt)
+    x@ptr <- x@ptr$classify(as.vector(rcl), NCOL(rcl), right, include.lowest, others, othersValue, bylayer[1], brackets[1], keepcats, opt)
 	messages(x, "classify")
 }
 )
@@ -488,8 +434,20 @@ function(x, from, to, others=NULL, raw=FALSE, filename="", ...) {
 			}
 		}
 		keepcats <- TRUE
-	} else if (fromc || toc) {
-		error("subst", "from or to has character values but x is not categorical")
+	} else {
+		if (fromc) {
+			error("subst", "from has character values but x is not categorical")
+		}
+		if (!tom) {
+			if (toc) {
+				to <- as.factor(to)
+				levels(x) <- data.frame(ID=1:length(levels(to)), value=levels(to))
+				keepcats <- TRUE
+			} else if (is.factor(to)) {
+				levels(x) <- data.frame(ID=1:length(levels(to)), value=levels(to))
+				keepcats <- TRUE			
+			}
+		}
 	}
 	
 	if (is.null(others)) {
@@ -503,11 +461,11 @@ function(x, from, to, others=NULL, raw=FALSE, filename="", ...) {
 	if (tom) {
 		nms <- colnames(to)
 		if (!is.null(nms)) 	opt$names = nms
-		x@cpp <- x@cpp$replaceValues(from, to, ncol(to), setothers, others, keepcats, opt)
+		x@ptr <- x@ptr$replaceValues(from, to, ncol(to), setothers, others, keepcats, opt)
 	} else if (frm) {
-		x@cpp <- x@cpp$replaceValues(as.vector(t(from)), to, -ncol(from), setothers, others, FALSE, opt)
+		x@ptr <- x@ptr$replaceValues(as.vector(t(from)), to, -ncol(from), setothers, others, keepcats, opt)
 	} else {
-		x@cpp <- x@cpp$replaceValues(from, to, 0, setothers, others, keepcats, opt)
+		x@ptr <- x@ptr$replaceValues(from, to, 0, setothers, others, keepcats, opt)
 	}
 	messages(x, "subst")
 }
@@ -534,14 +492,14 @@ setMethod("crop", signature(x="SpatRaster", y="ANY"),
 		opt <- spatOptions(filename, ...)
 		if (mask) {
 			if (inherits(y, "SpatVector")) {
-				x@cpp <- x@cpp$crop_mask(y@cpp, snap[1], touches[1], extend[1], opt)
+				x@ptr <- x@ptr$crop_mask(y@ptr, snap[1], touches[1], extend[1], opt)
 			} else if (inherits(y, "sf")) {
 				y <- vect(y)
-				x@cpp <- x@cpp$crop_mask(y@cpp, snap[1], touches[1], extend[1], opt)
+				x@ptr <- x@ptr$crop_mask(y@ptr, snap[1], touches[1], extend[1], opt)
 			} else if (inherits(y, "SpatRaster")) {
 				mopt <- spatOptions(filename="", ...)
 				e <- ext(y)
-				x@cpp <- x@cpp$crop(e@cpp, snap[1], extend[1], mopt)
+				x@ptr <- x@ptr$crop(e@ptr, snap[1], extend[1], mopt)
 				x <- messages(x, "crop")
 				if (compareGeom(x, y, lyrs=FALSE, crs=FALSE, warncrs=FALSE, ext=TRUE, rowcol=TRUE, res=FALSE, stopOnError=FALSE, messages=FALSE)) {
 					return(mask(x, y, filename=filename, ...))
@@ -556,11 +514,11 @@ setMethod("crop", signature(x="SpatRaster", y="ANY"),
 			} else {
 				y <- .getExt(y, method="crop")
 				warn("crop", paste("mask=TRUE is ignored for", class(y)[1], "objects"))
-				x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], opt)
+				x@ptr <- x@ptr$crop(y@ptr, snap[1], extend[1], opt)
 			}
 		} else {
 			y <- .getExt(y, method="crop")
-			x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], opt)
+			x@ptr <- x@ptr$crop(y@ptr, snap[1], extend[1], opt)
 		}
 		messages(x, "crop")
 	}
@@ -571,7 +529,7 @@ setMethod("crop", signature(x="SpatRasterDataset", y="ANY"),
 	function(x, y, snap="near", extend=FALSE) {
 		opt <- spatOptions()
 		y <- .getExt(y, method="crop")
-		x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], opt)
+		x@ptr <- x@ptr$crop(y@ptr, snap[1], extend[1], opt)
 		messages(x, "crop")
 	}
 )
@@ -580,7 +538,7 @@ setMethod("crop", signature(x="SpatRasterCollection", y="ANY"),
 	function(x, y, snap="near", extend=FALSE) {
 		y <- .getExt(y, method="crop")
 		opt <- spatOptions()
-		x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], double(), opt)
+		x@ptr <- x@ptr$crop(y@ptr, snap[1], extend[1], double(), opt)
 		messages(x, "crop")
 	}
 )
@@ -589,7 +547,7 @@ setMethod("crop", signature(x="SpatRasterCollection", y="ANY"),
 setMethod("selectRange", signature(x="SpatRaster"),
 	function(x, y, z=1, repint=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$selRange(y@cpp, z, repint, opt)
+		x@ptr <- x@ptr$selRange(y@ptr, z, repint, opt)
 		messages(x, "selectRange")
 	}
 )
@@ -597,7 +555,7 @@ setMethod("selectRange", signature(x="SpatRaster"),
 setMethod("cover", signature(x="SpatRaster", y="SpatRaster"),
 	function(x, y, values=NA, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$cover(y@cpp, values, opt)
+		x@ptr <- x@ptr$cover(y@ptr, values, opt)
 		messages(x, "cover")
 	}
 )
@@ -613,7 +571,7 @@ setMethod("diff", signature(x="SpatRaster"),
 		y <- x[[-((n-lag+1):n)]]
 		x <- x[[-(1:lag)]]
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$arith_rast(y@cpp, "-", FALSE, opt)
+		x@ptr <- x@ptr$arith_rast(y@ptr, "-", FALSE, opt)
 		messages(x, "diff")
 	}
 )
@@ -628,7 +586,7 @@ setMethod("disagg", signature(x="SpatRaster"),
 			return(r)
 		}
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$disaggregate(fact, opt)
+		x@ptr <- x@ptr$disaggregate(fact, opt)
 		messages(x, "disagg")
 	}
 )
@@ -638,7 +596,7 @@ setMethod("flip", signature(x="SpatRaster"),
 	function(x, direction="vertical", filename="", ...) {
 		d <- match.arg(direction, c("vertical", "horizontal"))
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$flip(d == "vertical", opt)
+		x@ptr <- x@ptr$flip(d == "vertical", opt)
 		messages(x, "flip")
 	}
 )
@@ -648,7 +606,7 @@ setMethod("flip", signature(x="SpatRaster"),
 setMethod("mask", signature(x="SpatRaster", mask="SpatRaster"),
 	function(x, mask, inverse=FALSE, maskvalues=NA, updatevalue=NA, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$mask_raster(mask@cpp, inverse[1], maskvalues, updatevalue[1], opt)
+		x@ptr <- x@ptr$mask_raster(mask@ptr, inverse[1], maskvalues, updatevalue[1], opt)
 		messages(x, "mask")
 	}
 )
@@ -656,11 +614,21 @@ setMethod("mask", signature(x="SpatRaster", mask="SpatRaster"),
 setMethod("mask", signature(x="SpatRaster", mask="SpatVector"),
 	function(x, mask, inverse=FALSE, updatevalue=NA, touches=TRUE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$mask_vector(mask@cpp, inverse[1], updatevalue[1], touches[1], opt)
+		x@ptr <- x@ptr$mask_vector(mask@ptr, inverse[1], updatevalue[1], touches[1], opt)
 		messages(x, "mask")
 	}
 )
 
+setMethod("mask", signature(x="SpatRaster", mask="SpatExtent"),
+	function(x, mask, inverse=FALSE, updatevalue=NA, touches=TRUE, filename="", ...) {
+		opt <- spatOptions(filename, ...)
+		mask <- vect(mask, crs=crs(x))
+		x@ptr <- x@ptr$mask_vector(mask@ptr, inverse[1], updatevalue[1], touches[1], opt)
+		messages(x, "mask")
+	}
+)
+
+
 setMethod("mask", signature(x="SpatRaster", mask="sf"),
    function(x, mask, inverse=FALSE, updatevalue=NA, touches=TRUE, filename="", ...) {
 		mask <- vect(mask)
@@ -669,10 +637,10 @@ setMethod("mask", signature(x="SpatRaster", mask="sf"),
 )
 
 setMethod("project", signature(x="SpatRaster"),
-	function(x, y, method, mask=FALSE, align=FALSE, res=NULL, origin=NULL, threads=FALSE, filename="", ..., use_gdal=TRUE, by_util = FALSE)  {
+	function(x, y, method, mask=FALSE, align_only=FALSE, res=NULL, origin=NULL, threads=FALSE, filename="", ..., use_gdal=TRUE, by_util = FALSE)  {
 
 		if (missing(method)) {
-			if (is.factor(x)[1] || isTRUE(x@cpp$rgb)) {
+			if (is.factor(x)[1] || isTRUE(x@ptr$rgb)) {
 				method <- "near"
 			} else {
 				method <- "bilinear"
@@ -690,15 +658,15 @@ setMethod("project", signature(x="SpatRaster"),
 			if (use_gdal) {
 
 				if (by_util) {
-						x@cpp <- x@cpp$warp_by_util(y@cpp, "", method, mask[1], align[1], FALSE, opt)
+						x@ptr <- x@ptr$warp_by_util(y@ptr, "", method, mask[1], align_only[1], FALSE, opt)
 				} else {
-					x@cpp <- x@cpp$warp(y@cpp, "", method, mask[1], align[1], FALSE, opt)
+					x@ptr <- x@ptr$warp(y@ptr, "", method, mask[1], align_only[1], FALSE, opt)
 				}
 			} else {
-				if (align) {
-					y <- project(rast(x), y, align=TRUE)
+				if (align_only) {
+					y <- project(rast(x), y, align_only=TRUE)
 				}
-				x@cpp <- x@cpp$resample(y@cpp, method, mask[1], TRUE, opt)
+				x@ptr <- x@ptr$resample(y@ptr, method, mask[1], TRUE, opt)
 			}
 		} else {
 			if (!is.character(y)) {
@@ -709,19 +677,19 @@ setMethod("project", signature(x="SpatRaster"),
 				tmp <- project(rast(x), y)
 				if (!is.null(res)) res(tmp) <- res
 				if (!is.null(origin)) origin(tmp) <- origin
-				return(project(x, tmp, method=method, mask=mask, align=align, filename=filename, use_gdal=use_gdal, by_util=by_util, ...))
+				return(project(x, tmp, method=method, mask=mask, align_only=align_only, filename=filename, use_gdal=use_gdal, by_util=by_util, ...))
 			}
 			if (use_gdal) {
 
 				if (by_util) {
-					x@cpp <- x@cpp$warp_by_util(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
+					x@ptr <- x@ptr$warp_by_util(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
 					
 				} else {
-					x@cpp <- x@cpp$warp(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
+					x@ptr <- x@ptr$warp(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
 				}
 			} else {
 				y <- project(rast(x), y)
-				x@cpp <- x@cpp$resample(y@cpp, method, mask[1], TRUE, opt)
+				x@ptr <- x@ptr$resample(y@ptr, method, mask[1], TRUE, opt)
 			}
 		}
 		messages(x, "project")
@@ -734,7 +702,7 @@ setMethod("project", signature(x="SpatVector"),
 		if (!is.character(y)) {
 			y <- as.character(crs(y))
 		}
-		x@cpp <- x@cpp$project(y, partial)
+		x@ptr <- x@ptr$project(y, partial)
 		messages(x, "project")
 	}
 )
@@ -767,9 +735,9 @@ setMethod("project", signature(x="matrix"),
            to <- as.character(crs(to))
         }
 		#v <- vect(x, type="line", crs=from)
-        #v@cpp <- v@cpp$project(to)
+        #v@ptr <- v@ptr$project(to)
         v <- vect()
-		xy <- v@cpp$project_xy(x[,1], x[,2], from, to)
+		xy <- v@ptr$project_xy(x[,1], x[,2], from, to)
 		messages(v, "project")
 		matrix(xy, ncol=2)
     }
@@ -779,7 +747,7 @@ setMethod("project", signature(x="matrix"),
 setMethod("quantile", signature(x="SpatRaster"),
 	function(x, probs=seq(0, 1, 0.25), na.rm=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$quantile(probs, na.rm[1], opt)
+		x@ptr <- x@ptr$quantile(probs, na.rm[1], opt)
 		messages(x, "quantile")
 	}
 )
@@ -815,7 +783,7 @@ setMethod("rectify", signature(x="SpatRaster"),
 			useaoi <- 0
 		}
 		snap <- as.logical(snap)
-		x@cpp <- x@cpp$rectify(method, aoi@cpp, useaoi, snap, opt)
+		x@ptr <- x@ptr$rectify(method, aoi@ptr, useaoi, snap, opt)
 		messages(x, "rectify")
 	}
 )
@@ -840,9 +808,9 @@ setMethod("resample", signature(x="SpatRaster", y="SpatRaster"),
 		}
 		opt <- spatOptions(filename, threads=threads, ...)
 #		if (gdal) {
-			x@cpp <- x@cpp$warp(y@cpp, "", method, FALSE, FALSE, TRUE, opt)
+			x@ptr <- x@ptr$warp(y@ptr, "", method, FALSE, FALSE, TRUE, opt)
 #		} else {
-#			x@cpp <- x@cpp$resample(y@cpp, method, FALSE, TRUE, opt)
+#			x@ptr <- x@ptr$resample(y@ptr, method, FALSE, TRUE, opt)
 #		}
 		messages(x, "resample")
 	}
@@ -855,7 +823,7 @@ setMethod("impose", signature(x="SpatRasterCollection"),
 		stopifnot(inherits(y, "SpatRaster"))
 		opt <- spatOptions(filename, ...)
 		r <- rast()
-		r@cpp <- x@cpp$morph(y@cpp, opt)
+		r@ptr <- x@ptr$morph(y@ptr, opt)
 		messages(r, "impose")
 	}
 )
@@ -865,7 +833,7 @@ setMethod("impose", signature(x="SpatRasterCollection"),
 setMethod("rev", signature(x="SpatRaster"),
 	function(x) {
 		opt <- spatOptions("", FALSE, list())
-		x@cpp <- x@cpp$reverse(opt)
+		x@ptr <- x@ptr$reverse(opt)
 		messages(x, "rev")
 	}
 )
@@ -873,7 +841,7 @@ setMethod("rev", signature(x="SpatRaster"),
 setMethod("rotate", signature(x="SpatRaster"),
 	function(x, left=TRUE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$rotate(left, opt)
+		x@ptr <- x@ptr$rotate(left, opt)
 		messages(x, "rotate")
 	}
 )
@@ -909,7 +877,7 @@ setMethod("rotate", signature(x="SpatVector"),
 			} 
 			x$unique_id_for_aggregation <- NULL
 		} else {
-			x@cpp <- x@cpp$rotate_longitude(longitude, left)
+			x@ptr <- x@ptr$rotate_longitude(longitude, left)
 			x <- messages(x, "rotate")
 		}
 		if (normalize) {
@@ -923,7 +891,7 @@ setMethod("segregate", signature(x="SpatRaster"),
 	function(x, classes=NULL, keep=FALSE, other=0, round=FALSE, digits=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		if (is.null(classes)) classes <- 1[0]
-		x@cpp <- x@cpp$separate(classes, keep[1], other[1], round[1], digits[1], opt)
+		x@ptr <- x@ptr$separate(classes, keep[1], other[1], round[1], digits[1], opt)
 		messages(x, "segregate")
 	}
 )
@@ -932,7 +900,7 @@ setMethod("segregate", signature(x="SpatRaster"),
 setMethod("shift", signature(x="SpatRaster"),
 	function(x, dx=0, dy=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$shift(dx, dy, opt)
+		x@ptr <- x@ptr$shift(dx, dy, opt)
 		messages(x, "shift")
 	}
 )
@@ -948,7 +916,7 @@ setMethod("shift", signature(x="SpatExtent"),
 
 setMethod("shift", signature(x="SpatVector"),
 	function(x, dx=0, dy=0) {
-		x@cpp <- x@cpp$shift(dx, dy)
+		x@ptr <- x@ptr$shift(dx, dy)
 		messages(x, "shift")
 	}
 )
@@ -966,7 +934,7 @@ setMethod("rescale", signature(x="SpatRaster"),
 		}
 		ex = x0 + fx * (e[1:2] - x0);
 		ey = y0 + fy * (e[3:4] - y0);
-		x@cpp <- x@cpp$deepcopy()
+		x@ptr <- x@ptr$deepcopy()
 		ext(x) <- ext(c(ex, ey))
 		messages(x, "rescale")
 	}
@@ -983,7 +951,7 @@ setMethod("rescale", signature(x="SpatVector"),
 		if (missing(y0)) {
 			y0 <- mean(e[3:4])
 		}
-		x@cpp <- x@cpp$rescale(fx, fy, x0[1], y0[1])
+		x@ptr <- x@ptr$rescale(fx, fy, x0[1], y0[1])
 		messages(x, "rescale")
 	}
 )
@@ -1006,7 +974,7 @@ setMethod("scale", signature(x="SpatRaster"),
 		} else {
 			doscale = TRUE;
 		}
-		x@cpp <- x@cpp$scale(center, docenter, scale, doscale, opt)
+		x@ptr <- x@ptr$scale(center, docenter, scale, doscale, opt)
 		messages(x, "scale")
 	}
 )
@@ -1016,9 +984,11 @@ setMethod("scale", signature(x="SpatRaster"),
 setMethod("stretch", signature(x="SpatRaster"),
 	function(x, minv=0, maxv=255, minq=0, maxq=1, smin=NA, smax=NA, histeq=FALSE, scale=1, maxcell=500000, filename="", ...) {
 		if (histeq) {
+			nms <- names(x)
 			if (nlyr(x) > 1) {
 				x <- lapply(1:nlyr(x), function(i) stretch(x[[i]], histeq=TRUE, scale=scale, maxcell=maxcell))
 				x <- rast(x)
+				names(x) <- nms 
 				if (filename != "") {
 					x <- writeRaster(x, filename=filename, ...)
 				}
@@ -1033,10 +1003,14 @@ setMethod("stretch", signature(x="SpatRaster"),
 					f(y) * scale
 				}
 			}
-			app(x, ecdfun, filename=filename, ...)
+			wopt <- list(...)
+			if (is.null(wopt$names)) {
+				wopt$names <- nms
+			}
+			app(x, ecdfun, filename=filename, wopt=wopt)
 		} else {
 			opt <- spatOptions(filename, ...)
-			x@cpp <- x@cpp$stretch(minv, maxv, minq, maxq, smin, smax, opt)
+			x@ptr <- x@ptr$stretch(minv, maxv, minq, maxq, smin, smax, opt)
 			messages(x, "stretch")
 		}
 	}
@@ -1069,14 +1043,14 @@ setMethod("summary", signature(object="SpatVector"),
 setMethod("t", signature(x="SpatRaster"),
 	function(x) {
 		opt <- spatOptions()
-		x@cpp <- x@cpp$transpose(opt)
+		x@ptr <- x@ptr$transpose(opt)
 		messages(x, "t")
 	}
 )
 
 setMethod("t", signature(x="SpatVector"),
 	function(x) {
-		x@cpp <- x@cpp$transpose()
+		x@ptr <- x@ptr$transpose()
 		messages(x, "t")
 	}
 )
@@ -1087,7 +1061,7 @@ setMethod("terrain", signature(x="SpatRaster"),
 		unit <- match.arg(unit, c("degrees", "radians"))
 		opt <- spatOptions(filename, ...)
 		seed <- ifelse("flowdir" %in% v, .seed(), 0)
-		x@cpp <- x@cpp$terrain(v, neighbors[1], unit=="degrees", seed, opt)
+		x@ptr <- x@ptr$terrain(v, neighbors[1], unit=="degrees", seed, opt)
 		messages(x, "terrain")
 	}
 )
@@ -1103,7 +1077,7 @@ setMethod("viewshed", signature(x="SpatRaster"),
 		outops <- c("yes/no", "sea", "land")
 		output <- match.arg(tolower(output), outops)
 		output <- match(output, outops)
-		z@cpp <- x@cpp$view(c(loc[1:2], observer[1], target[1]), c(1,0,2,3), curvcoef, 2, 0, output, opt)
+		z@ptr <- x@ptr$view(c(loc[1:2], observer[1], target[1]), c(1,0,2,3), curvcoef, 2, 0, output, opt)
 		messages(z, "viewshed")
 	}
 )
@@ -1111,7 +1085,7 @@ setMethod("viewshed", signature(x="SpatRaster"),
 setMethod("sieve", signature(x="SpatRaster"),
 	function(x, threshold, directions=8, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$sieve(threshold[1], directions[1], opt)
+		x@ptr <- x@ptr$sieve(threshold[1], directions[1], opt)
 		messages(x, "sieve")
 	}
 )
@@ -1124,7 +1098,7 @@ setMethod("trim", signature(x="SpatRaster"),
 		if (padding < 0) {
 			error("trim", "padding must be a non-negative integer")
 		}
-		x@cpp <- x@cpp$trim(value[1], padding, opt)
+		x@ptr <- x@ptr$trim(value[1], padding, opt)
 		messages(x, "trim")
 	}
 )
@@ -1132,7 +1106,7 @@ setMethod("trim", signature(x="SpatRaster"),
 setMethod("trans", signature(x="SpatRaster"),
 	function(x, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$transpose(opt)
+		x@ptr <- x@ptr$transpose(opt)
 		messages(x, "trans")
 	}
 )
@@ -1144,7 +1118,7 @@ setMethod("unique", signature(x="SpatRaster", incomparables="ANY"),
 		opt <- spatOptions()
 
 		if (as.raster) incomparables = FALSE
-		u <- x@cpp$unique(incomparables, digits, na.rm[1], opt)
+		u <- x@ptr$unique(incomparables, digits, na.rm[1], opt)
 
 		if (!as.raster) {
 			u <- get_labels(x, u)
@@ -1202,7 +1176,7 @@ setMethod("labels", signature(object="SpatRaster"),
 
 setMethod("scoff", signature(x="SpatRaster"),
 	function(x) {
-		out <- x@cpp$getScaleOffset()
+		out <- x@ptr$getScaleOffset()
 		names(out) <- c("scale", "offset")
 		do.call(cbind, out)
 	}
@@ -1211,17 +1185,17 @@ setMethod("scoff", signature(x="SpatRaster"),
 setMethod("scoff<-", signature("SpatRaster"),
 	function(x, value) {
 		if (is.null(value)) {
-			x@cpp <- x@cpp$deepcopy()
-			x@cpp$setScaleOffset(1, 0)
+			x@ptr <- x@ptr$deepcopy()
+			x@ptr$setScaleOffset(1, 0)
 		} else {
 			if (NCOL(value) != 2) {
 				error("scoff<-", "value must be a 2-column matrix")
 			}
-			x@cpp <- x@cpp$deepcopy()
+			x@ptr <- x@ptr$deepcopy()
 			value[is.na(value[,1]),1] <- 1
 			value[is.na(value[,2]),2] <- 0
-			x@cpp$setScaleOffset(value[,1], value[,2])
-			x@cpp$setValueType(0)
+			x@ptr$setScaleOffset(value[,1], value[,2])
+			x@ptr$setValueType(0)
 		}
 		messages(x, "scoff<-")
 	}
@@ -1230,7 +1204,7 @@ setMethod("scoff<-", signature("SpatRaster"),
 setMethod("sort", signature(x="SpatRaster"),
 	function (x, decreasing=FALSE, order=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$sort(decreasing[1], order[1], opt)
+		x@ptr <- x@ptr$sort(decreasing[1], order[1], opt)
 		messages(x, "sort")
 	}
 )
