@@ -2,7 +2,7 @@
 setMethod("droplevels", signature(x="SpatRaster"),
 	function(x, level=NULL, layer=1) {
 		if (is.null(level)) {
-			x@cpp <- x@cpp$droplevels()
+			x@ptr <- x@ptr$droplevels()
 			messages(x)
 		} else {
 			if (is.character(layer)) {
@@ -12,7 +12,7 @@ setMethod("droplevels", signature(x="SpatRaster"),
 				}
 			}
 			x[[layer]][x[[layer]] %in%  level] <- NA
-			x@cpp <- x@cpp$droplevels()
+			x@ptr <- x@ptr$droplevels()
 			messages(x)
 		}
 	}
@@ -21,13 +21,13 @@ setMethod("droplevels", signature(x="SpatRaster"),
 
 setMethod("is.factor", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$hasCategories()
+		x@ptr$hasCategories()
 	}
 )
 
 setMethod("as.factor", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp = x@cpp$makeCategorical(-1, spatOptions())
+		x@ptr = x@ptr$makeCategorical(-1, spatOptions())
 		messages(x)
 		#if (!hasValues(x)) {
 		#	error("as.factor", "x has no values")
@@ -44,7 +44,7 @@ setMethod("as.factor", signature(x="SpatRaster"),
 
 setMethod("levels", signature(x="SpatRaster"),
 	function(x) {
-		x <- x@cpp$getCategories()
+		x <- x@ptr$getCategories()
 		lapply(x, function(i) {
 			d <- .getSpatDF(i$df)
 			if (ncol(d) == 0) return("")
@@ -56,9 +56,9 @@ setMethod("levels", signature(x="SpatRaster"),
 
 setMethod("levels<-", signature(x="SpatRaster"),
 	function(x, value) {
-		x@cpp <- x@cpp$deepcopy()
+		x@ptr <- x@ptr$deepcopy()
 		if (is.null(value)) {
-			x@cpp$removeCategories(-1)
+			x@ptr$removeCategories(-1)
 			return(messages(x, "levels<-"))
 		} else if (inherits(value, "list")) {
 			for (i in 1:length(value)) {
@@ -78,7 +78,7 @@ setMethod ("set.cats" , "SpatRaster",
 
 		if (missing(value)) {
 			error("set.cats", "value cannot be missing")
-			#return(invisible(x@cpp$setCatIndex(layer-1, index)))
+			#return(invisible(x@ptr$setCatIndex(layer-1, index)))
 		}
 
 		if (is.character(layer)) {
@@ -134,7 +134,7 @@ setMethod ("set.cats" , "SpatRaster",
 			value <- value[[1]]
 		}
 		if (is.null(value)) {
-			x@cpp$removeCategories(layer-1)
+			x@ptr$removeCategories(layer-1)
 			messages(x, "set.cats")
 			return(invisible(TRUE))
 		}
@@ -177,7 +177,7 @@ setMethod ("set.cats" , "SpatRaster",
 			cn <- colnames(value)[index+1]
 			if (!(tolower(cn) %in% c("histogram", "count", "red", "green", "blue", "alpha", "opacity", "r", "g", "b", "a"))) {
 				nms[layer] <- cn
-				if (! x@cpp$setNames(nms, FALSE)) {
+				if (! x@ptr$setNames(nms, FALSE)) {
 					error("names<-", "cannot set name")
 				}
 			}
@@ -190,7 +190,7 @@ setMethod ("set.cats" , "SpatRaster",
 		}
 
 		value <- .makeSpatDF(value)
-		ok <- x@cpp$setCategories(layer-1, value, index)
+		ok <- x@ptr$setCategories(layer-1, value, index)
 		x <- messages(x, "set.cats")
 		invisible(ok)
 	}
@@ -201,7 +201,7 @@ setMethod ("set.cats" , "SpatRaster",
 setMethod ("categories" , "SpatRaster",
 	function(x, layer=1, value, active=1, ...) {
 		#... to accept but ignore old argument "index"
-		x@cpp <- x@cpp$deepcopy()
+		x@ptr <- x@ptr$deepcopy()
 		set.cats(x, layer, value, active)
 		x
 	}
@@ -218,12 +218,12 @@ setMethod ("activeCat" , "SpatRaster",
 			}
 		}
 		if (layer < 1) {
-			sapply(1:nlyr(x), function(i) x@cpp$getCatIndex(i-1))
+			sapply(1:nlyr(x), function(i) x@ptr$getCatIndex(i-1))
 		} else {
 			if (!is.factor(x)[layer]) {
 				return(NA)
 			}
-			x@cpp$getCatIndex(layer-1)
+			x@ptr$getCatIndex(layer-1)
 		}
 	}
 )
@@ -250,7 +250,7 @@ setMethod("activeCat<-" , "SpatRaster",
 			}
 		}
 		x <- deepcopy(x)
-		if (!x@cpp$setCatIndex(layer-1, value)) {
+		if (!x@ptr$setCatIndex(layer-1, value)) {
 			error("activeCat", "invalid category index")
 		}
 		x
@@ -262,7 +262,7 @@ setMethod("cats" , "SpatRaster",
 		if (!missing(layer)) {
 			x <- subset(x, layer, NSE=FALSE)
 		}
-		cats <- x@cpp$getCategories()
+		cats <- x@ptr$getCategories()
 		lapply(1:nlyr(x), function(i) {
 			if (cats[[i]]$df$nrow == 0) {
 				return(NULL)
@@ -279,7 +279,7 @@ setMethod("cats" , "SpatRaster",
 	if (!any(ff)) {
 		return (lapply(ff, function(i) NULL))
 	}
-	cats <- x@cpp$getCategories()
+	cats <- x@ptr$getCategories()
 	x <- lapply(1:length(cats), function(i) {
 		if (cats[[i]]$df$nrow == 0) return(NULL)
 		r <- .getSpatDF(cats[[i]]$df)
@@ -300,7 +300,7 @@ setMethod ("as.numeric", "SpatRaster",
 	function(x, index=NULL, filename="", ...) {
 		if (!any(is.factor(x))) {
 			x <- deepcopy(x)
-			x@cpp$setValueType(0)
+			x@ptr$setValueType(0)
 			return(x)
 		}
 		if (nlyr(x) > 1) {
@@ -320,7 +320,7 @@ setMethod ("as.numeric", "SpatRaster",
 				}
 				if (index == 1) {
 					levels(x) <- NULL
-					x@cpp$setValueType(0)
+					x@ptr$setValueType(0)
 					if (filename != "") {
 						x <- writeRaster(x, filename, ...)
 					}
@@ -337,7 +337,7 @@ setMethod ("as.numeric", "SpatRaster",
 			index <- activeCat(x, 1)
 			if (index <= 1) {
 				levels(x) <- NULL
-				x@cpp$setValueType(0)
+				x@ptr$setValueType(0)
 				if (filename != "") {
 					x <- writeRaster(x, filename, ...)
 				}
@@ -426,7 +426,7 @@ setMethod("catalyze", "SpatRaster",
 setMethod("concats", "SpatRaster",
 	function(x, y, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp = x@cpp$combineCats(y@cpp, opt)
+		x@ptr = x@ptr$combineCats(y@ptr, opt)
 		messages(x, "concats")
 	}
 )
