@@ -32,7 +32,7 @@ new_rast <- function(nrows=10, ncols=10, nlyrs=1, xmin=0, xmax=1, ymin=0, ymax=1
 	#check_proj4_datum(crs)
 
 	r <- methods::new("SpatRaster")
-	r@cpp <- SpatRaster$new(c(nrows, ncols, nlyrs), e, crs)
+	r@ptr <- SpatRaster$new(c(nrows, ncols, nlyrs), e, crs)
 	r <- messages(r, "rast")
 
 	if (!missing(resolution)) {
@@ -72,11 +72,49 @@ setMethod("rast", signature(x="missing"),
 )
 
 
+
+
+rast_from_image <- function(x) { 
+	# list representing an "image"
+	if (! all(dim(x$z) == c(length(x$x), length(x$y)))) {
+		error("rast", '"z" does not have the right dimensions') 
+	}
+	# omitted "-1" bug fix by Barry Rowlingson 
+	resx <- ( x$x[length(x$x)] - x$x[1] ) / (length(x$x)-1)
+	resy <- ( x$y[length(x$y)] - x$y[1] ) / (length(x$y)-1)
+	xmn <- min(x$x) - 0.5 * resx
+	xmx <- max(x$x) + 0.5 * resx
+	ymn <- min(x$y) - 0.5 * resy
+	ymx <- max(x$y) + 0.5 * resy
+
+	dx <- abs(max(abs((x$x[-1] - x$x[-length(x$x)])) / resx) - 1)
+	dy <- abs(max(abs((x$y[-1] - x$y[-length(x$y)])) / resy) - 1)
+	if (is.na(dx) | is.na(dy)) {
+		error("rast", "missing values in coordinates")
+	} 
+	if (dx > 0.01 | dy > 0.01) {
+		error("rast", "data are not on a regular grid")
+	}
+#	if (xmn > -360.1 & xmx < 360.1 & ymn > -90.1 & ymx < 90.1) { 
+#		crs <- "+proj=longlat +datum=WGS84"
+#	} else {
+#		crs <- ""
+#	}
+	x <- t(x$z)
+	x <- x[nrow(x):1, ]
+	rast(x, extent=ext(xmn, xmx, ymn, ymx), crs="")
+}
+
+
+
 setMethod("rast", signature(x="list"),
 	function(x, warn=TRUE) {
 		i <- sapply(x, function(i) inherits(i, "SpatRaster"))
 		if (!all(i)) {
 			if (!any(i)) {
+				if ((length(x) == 3) && all(c("x", "y", "z") %in% names(x))) {
+					return(rast_from_image(x))
+				}
 				error("rast,list", "none of the elements of x are a SpatRaster")
 			} else {
 				warn("rast", sum(!i), " out of ", length(x), " elements of x are not a SpatRaster")
@@ -90,7 +128,7 @@ setMethod("rast", signature(x="list"),
 		}
 		opt <- spatOptions()
 		for (i in 2:length(x)) {
-			out@cpp$addSource(x[[i]]@cpp, warn, opt)
+			out@ptr$addSource(x[[i]]@ptr, warn, opt)
 		}
 		out <- messages(out, "rast")
 		lnms <- names(x)
@@ -200,13 +238,13 @@ setMethod("rast", signature(x="character"),
 		if (is.null(drivers)) drivers <- ""[0]
 		if (length(subds) == 0) subds = 0
 		if (is.character(subds)) {
-			#r@cpp <- SpatRaster$new(f, -1, subds, FALSE, 0[])
-			r@cpp <- SpatRaster$new(f, -1, subds, FALSE, drivers, opts, 0[])
+			#r@ptr <- SpatRaster$new(f, -1, subds, FALSE, 0[])
+			r@ptr <- SpatRaster$new(f, -1, subds, FALSE, drivers, opts, 0[])
 		} else {
-			r@cpp <- SpatRaster$new(f, subds-1, "", FALSE, drivers, opts, 0[])
+			r@ptr <- SpatRaster$new(f, subds-1, "", FALSE, drivers, opts, 0[])
 		}
 		r <- messages(r, "rast")
-		if (r@cpp$getMessage() == "ncdf extent") {
+		if (r@ptr$getMessage() == "ncdf extent") {
 			# could have used opts="IGNORE_XY_AXIS_NAME_CHECKS=YES"
 			test <- try(r <- .ncdf_extent(r, f), silent=TRUE)
 			if (inherits(test, "try-error")) {
@@ -250,11 +288,11 @@ multi <- function(x, subds=0, xyz=3:1, drivers=NULL, opts=NULL) {
 	subds <- subds[1]
 
 	if (is.character(subds)) {
-		r@cpp <- SpatRaster$new(f, -1, subds, TRUE, drivers, opts, xyz-1)
+		r@ptr <- SpatRaster$new(f, -1, subds, TRUE, drivers, opts, xyz-1)
 	} else {
-		r@cpp <- SpatRaster$new(f, subds-1, ""[0], TRUE, drivers, opts, xyz-1)
+		r@ptr <- SpatRaster$new(f, subds-1, ""[0], TRUE, drivers, opts, xyz-1)
 	}
-	if (r@cpp$getMessage() == "ncdf extent") {
+	if (r@ptr$getMessage() == "ncdf extent") {
 		test <- try(r <- .ncdf_extent(r), silent=TRUE)
 		if (inherits(test, "try-error")) {
 			warn("rast", "GDAL did not find an extent. Cells not equally spaced?")
@@ -276,7 +314,7 @@ setMethod("rast", signature(x="SpatRaster"),
 		if (inherits(nlyrs, "SpatRaster")) {
 			error("rast", "use 'c()' to combine SpatRasters")
 		}
-		x@cpp <- x@cpp$geometry(nlyrs, props, keeptime, keepunits, tags)
+		x@ptr <- x@ptr$geometry(nlyrs, props, keeptime, keepunits, tags)
 		x <- messages(x, "rast")
 		if (!missing(names)) {
 			if (length(names) == nlyr(x)) names(x) <- names
@@ -297,7 +335,7 @@ setMethod("rast", signature(x="SpatRasterDataset"),
 			x[1]
 		} else {
 			r <- methods::new("SpatRaster")
-			r@cpp <- x@cpp$collapse()
+			r@ptr <- x@ptr$collapse()
 			nms <- names(x)
 			if (any(nms != "")) {
 				names(r) <- paste(rep(nms, nlyr(x)), names(r), sep="_")
@@ -332,7 +370,7 @@ setMethod("rast", signature(x="array"),
 			e <- c(0, dims[2], 0, dims[1])
 		}
 		crs <- character_crs(crs, "rast")
-		r@cpp <- SpatRaster$new(dims, e, crs)
+		r@ptr <- SpatRaster$new(dims, e, crs)
 		values(r) <- x
 		messages(r, "rast")
 	}
@@ -526,7 +564,7 @@ setMethod("rast", signature(x="stars_proxy"),
 setMethod("NAflag<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		value <- as.numeric(value)
-		if (!(x@cpp$setNAflag(value))) {
+		if (!(x@ptr$setNAflag(value))) {
 			error("NAflag<-", "cannot set this value")
 		}
 		x
@@ -535,7 +573,7 @@ setMethod("NAflag<-", signature(x="SpatRaster"),
 
 setMethod("NAflag", signature(x="SpatRaster"),
 	function(x)  {
-		x@cpp$getNAflag()
+		x@ptr$getNAflag()
 	}
 )
 
