@@ -59,9 +59,9 @@ setMethod("zonal", signature(x="SpatRaster", z="SpatRaster"),
 				if (txtfun != "mean") {
 					error("zonal", "fun must be 'mean' when using weights")
 				}
-				sdf <- x@cpp$zonal_weighted(z@cpp, w@cpp, na.rm, opt)			
+				sdf <- x@ptr$zonal_weighted(z@ptr, w@ptr, na.rm, opt)			
 			} else {
-				sdf <- x@cpp$zonal(z@cpp, grast@cpp, txtfun, na.rm, opt)
+				sdf <- x@ptr$zonal(z@ptr, grast@ptr, txtfun, na.rm, opt)
 			}
 			sdf <- messages(sdf, "zonal")
 			out <- .getSpatDF(sdf)
@@ -162,23 +162,55 @@ setMethod("zonal", signature(x="SpatRaster", z="SpatRaster"),
 
 
 setMethod("zonal", signature(x="SpatRaster", z="SpatVector"),
-	function(x, z, fun="mean", ..., w=NULL, weights=FALSE, exact=FALSE, touches=FALSE, as.raster=FALSE, filename="", wopt=list())  {
+	function(x, z, fun="mean", na.rm=FALSE, w=NULL, weights=FALSE, exact=FALSE, touches=FALSE, small=TRUE, as.raster=FALSE, as.polygons=FALSE, wide=TRUE, filename="", wopt=list())  {
 		opt <- spatOptions()
-		narm <- isTRUE(list(...)$na.rm)
 		txtfun <- .makeTextFun(fun)
 		if (!inherits(txtfun, "character")) {
 			error("zonal", "this 'fun' is not supported. You can use extract instead")
 		} else {
-			if (is.null(w)) {
-				out <- x@cpp$zonal_poly(z@cpp, txtfun, weights[1], exact[1], touches[1], narm, opt)
-			} else {
-				if (txtfun != "mean") {
-					error("zonal", "fun must be 'mean' when using weights")
+			if (txtfun == "table") {
+				if (!is.null(w)) {
+					error("cannot use 'w' when 'fun=table'")
 				}
-				out <- x@cpp$zonal_poly_weighted(z@cpp, w@cpp, weights[1], exact[1], touches[1], narm, opt)
+				v <- x@ptr$zonal_poly_table(z@ptr, weights[1], exact[1], touches[1], small[1], na.rm, opt)
+				messages(x, "zonal")
+
+				v <- lapply(v, function(i) if (length(i) == 0) NA else i)
+				v <- lapply(1:length(v), function(i) cbind(i, matrix(v[[i]], ncol=2)))
+				v <- do.call(rbind, v)
+				v <- as.data.frame(v)
+				colnames(v) <- c("zone", "value", "count")
+				ff <- is.factor(x)[1]
+				if (ff) {
+					cg <- cats(x)[[1]]
+					i <- match(v$value, cg[,1])
+					act <- activeCat(x, 1) + 1
+					v$value <- cg[i, act]
+				}
+				if (as.polygons | wide) {
+					nms <- names(v)
+					v <- stats::reshape(v, direction="wide", idvar=nms[1], timevar=nms[2])
+					names(v) <- gsub("count.", "", names(v))
+					v[is.na(v)] <- 0
+					rownames(v) <- NULL
+				}
+				if (as.polygons) {
+					values(z) <- v
+					return(z)
+				}
+				return(v)
+			} else {
+				if (is.null(w)) {
+					out <- x@ptr$zonal_poly(z@ptr, txtfun, weights[1], exact[1], touches[1], small[1], na.rm, opt)
+				} else {
+					if (txtfun != "mean") {
+						error("zonal", "fun must be 'mean' when using weights")
+					}
+					out <- x@ptr$zonal_poly_weighted(z@ptr, w@ptr, weights[1], exact[1], touches[1], small[1], na.rm, opt)
+				}
+				messages(out, "zonal")
+				out <- .getSpatDF(out)
 			}
-			messages(out, "zonal")
-			out <- .getSpatDF(out)
 		}
 		if (as.raster) {
 			if (is.null(wopt$names)) {
@@ -186,6 +218,8 @@ setMethod("zonal", signature(x="SpatRaster", z="SpatVector"),
 			}
 			x <- rasterize(z, x, 1:nrow(z))
 			subst(x, 1:nrow(out), out, filename=filename, wopt=wopt)
+		} else if (as.polygons) {
+			cbind(z, out)	
 		} else {
 			out
 		}
@@ -271,7 +305,7 @@ setMethod("global", signature(x="SpatRaster"),
 			stopifnot(inherits(weights, "SpatRaster"))
 			stopifnot(txtfun %in% c("mean", "sum"))
 			na.rm <- isTRUE(list(...)$na.rm)
-			ptr <- x@cpp$global_weighted_mean(weights@cpp, txtfun, na.rm, opt)
+			ptr <- x@ptr$global_weighted_mean(weights@ptr, txtfun, na.rm, opt)
 			messages(ptr, "global")
 			res <- .getSpatDF(ptr)
 			rownames(res) <- nms
@@ -289,11 +323,11 @@ setMethod("global", signature(x="SpatRaster"),
 				}
 				txtfun <- unique(txtfun)
 				na.rm <- isTRUE(list(...)$na.rm)
-				if (isTRUE(list(...)$old)) {
-					ptr <- x@cpp$global(txtfun, na.rm, opt)			
-				} else {
-					ptr <- x@cpp$mglobal(txtfun, na.rm, opt)
-				}
+				#if (isTRUE(list(...)$old)) {
+				#	ptr <- x@ptr$global(txtfun, na.rm, opt)			
+				#} else {
+				ptr <- x@ptr$mglobal(txtfun, na.rm, opt)
+				#}
 				messages(ptr, "global")
 				res <- .getSpatDF(ptr)
 				rownames(res) <- nms
@@ -405,9 +439,9 @@ setMethod("freq", signature(x="SpatRaster"),
 			}
 
 			if (is.na(digits)) {
-				v <- x@cpp$count(value, bylayer[1], FALSE, 0, opt)
+				v <- x@ptr$count(value, bylayer[1], FALSE, 0, opt)
 			} else {
-				v <- x@cpp$count(value, bylayer[1], TRUE, digits, opt)
+				v <- x@ptr$count(value, bylayer[1], TRUE, digits, opt)
 				value <- round(value, digits)
 			}
 			if (bylayer) {
@@ -418,9 +452,9 @@ setMethod("freq", signature(x="SpatRaster"),
 
 		} else {
 			if (is.na(digits)) {
-				v <- x@cpp$freq(bylayer[1], FALSE, 0, opt)
+				v <- x@ptr$freq(bylayer[1], FALSE, 0, opt)
 			} else {
-				v <- x@cpp$freq(bylayer[1], TRUE, digits, opt)
+				v <- x@ptr$freq(bylayer[1], TRUE, digits, opt)
 			}
 			v <- lapply(v, function(i) if (length(i) == 0) NA else i)
 
@@ -473,14 +507,14 @@ setMethod("freq", signature(x="SpatRaster"),
 
 
 setMethod ("expanse", "SpatRaster",
-	function(x, unit="m", transform=TRUE, byValue=FALSE, zones=NULL, wide=FALSE) {
+	function(x, unit="m", transform=TRUE, byValue=FALSE, zones=NULL, wide=FALSE, usenames=FALSE) {
 		opt <- spatOptions()
 		if (!is.null(zones)) {
 			if (!inherits(zones, "SpatRaster")) {
 				error("expanse", "zones must be a SpatRaster")
 			}
 			compareGeom(x, zones, lyrs=FALSE, crs=FALSE, ext=TRUE, rowcol=TRUE)
-			v <- x@cpp$sum_area_group(zones@cpp, unit[1], transform[1], byValue[1], opt)
+			v <- x@ptr$sum_area_group(zones@ptr, unit[1], transform[1], byValue[1], opt)
 			messages(x)
 			v <- lapply(v, function(i) matrix(i, ncol=4, byrow=TRUE))
 			v <- data.frame(do.call(rbind, v))
@@ -503,9 +537,8 @@ setMethod ("expanse", "SpatRaster",
 				}
 				v[is.na(v)] <- 0
 			}
-			return(v)
 		} else {
-			v <- x@cpp$sum_area(unit, isTRUE(transform[1]), isTRUE(byValue[1]), opt)
+			v <- x@ptr$sum_area(unit, isTRUE(transform[1]), isTRUE(byValue[1]), opt)
 			x <- messages(x, "expanse")
 			if (byValue) {
 				v <- lapply(1:length(v), function(i) cbind(i, matrix(v[[i]], ncol=2, byrow=TRUE)))
@@ -523,8 +556,11 @@ setMethod ("expanse", "SpatRaster",
 				}
 				v[is.na(v)] <- 0
 			}
-			v
 		}
+		if (usenames) {
+			v$layer <- names(x)[v$layer]
+		}
+		v
 	}
 )
 
