@@ -101,7 +101,7 @@ bool SpatRaster::isSource(std::string filename) {
 
 SpatRaster SpatRaster::writeRaster(SpatOptions &opt) {
 
-	SpatRaster out = geometry_opt(nlyr(), true, true, true, true, opt);
+	SpatRaster out = geometry_opt(nlyr(), true, true, true, true, true, opt);
 	if (!hasValues()) {
 		out.setError("there are no cell values");
 		return out;
@@ -164,7 +164,7 @@ SpatRaster SpatRaster::writeRaster(SpatOptions &opt) {
 
 SpatRaster SpatRaster::writeTempRaster(SpatOptions &opt) {
 	SpatOptions xopt(opt);
-	std::string fname = tempFile(xopt.get_tempdir(), xopt.pid, "_temp_raster.tif");
+	std::string fname = tempFile(xopt.get_tempdir(), xopt.tmpfile, "_temp_raster.tif");
 	xopt.set_filenames({fname});
 	return writeRaster(xopt);
 }
