@@ -570,6 +570,13 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 	std::vector<std::string> nms = getNames();
 	double naflag=NAN;
 	bool hasNAflag = opt.has_NAflag(naflag);
+	
+//	if (driver == "AAIGrid" && std::isnan(naflag)) {
+		// avoid nan as flag
+//		naflag = -3.40282347E+38;
+//		hasNAflag = true;
+// set opt.NAflag?
+//	}
 
 	if (writeRGB) nms = {"red", "green", "blue"};
 
@@ -645,6 +652,16 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 		*/
 		poBand->SetDescription(nms[i].c_str());
 
+		if (driver == "GTiff") {
+			std::vector<std::string> m = getLyrTags({i});
+			if (m.size() > 0) {
+				for (size_t i=0; i<m.size(); i+=2) {
+					poBand->SetMetadataItem(m[i].c_str(), m[i+1].c_str());
+				}
+			}
+		}
+
+
 		if ((i==0) || (driver != "GTiff")) {
 			// to avoid "Setting nodata to nan on band 2, but band 1 has nodata at nan."
 			if (hasNAflag) {
